#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Drawing;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.GeneralizedQuadraticAssignment {
  [Item("Generalized Quadratic Assignment Problem", "The Generalized Quadratic Assignment Problem (GQAP) is a generalization of the QAP in that it allows to assign multiple facilities (here called equipment) to a single location. The problem is described in Lee, C.G., and Ma, Z. 2003. The Generalized Quadratic Assignment Problem. Technical Report M5S 3G8, University of Toronto, Canada.")]
  [Creatable("Problems")]
  [StorableClass]
  public sealed class GeneralizedQuadraticAssignmentProblem : SingleObjectiveHeuristicOptimizationProblem<IGQAPEvaluator, IIntegerVectorCreator> {
    public override Image ItemImage {
      get { return HeuristicLab.Common.Resources.VSImageLibrary.Type; }
    }

    #region Parameter Properties
    public ValueParameter<DoubleMatrix> WeightsParameter {
      get { return (ValueParameter<DoubleMatrix>)Parameters["Weights"]; }
    }
    public ValueParameter<DoubleMatrix> DistancesParameter {
      get { return (ValueParameter<DoubleMatrix>)Parameters["Distances"]; }
    }
    public ValueParameter<DoubleMatrix> InstallationCostsParameter {
      get { return (ValueParameter<DoubleMatrix>)Parameters["InstallationCosts"]; }
    }
    public FixedValueParameter<DoubleValue> TransportationCostsParameter {
      get { return (FixedValueParameter<DoubleValue>)Parameters["TransportationCosts"]; }
    }
    public ValueParameter<DoubleArray> DemandsParameter {
      get { return (ValueParameter<DoubleArray>)Parameters["Demands"]; }
    }
    public ValueParameter<DoubleArray> CapacitiesParameter {
      get { return (ValueParameter<DoubleArray>)Parameters["Capacities"]; }
    }
    public OptionalValueParameter<IItem> BestKnownSolutionParameter {
      get { return (OptionalValueParameter<IItem>)Parameters["BestKnownSolution"]; }
    }

    #endregion

    #region Properties
    public DoubleMatrix Weights {
      get { return WeightsParameter.Value; }
      set { WeightsParameter.Value = value; }
    }
    public DoubleMatrix Distances {
      get { return DistancesParameter.Value; }
      set { DistancesParameter.Value = value; }
    }
    public DoubleMatrix InstallationCosts {
      get { return InstallationCostsParameter.Value; }
      set { InstallationCostsParameter.Value = value; }
    }
    public DoubleValue TransportationCosts {
      get { return TransportationCostsParameter.Value; }
      set { TransportationCostsParameter.Value = value; }
    }
    public DoubleArray Demands {
      get { return DemandsParameter.Value; }
      set { DemandsParameter.Value = value; }
    }
    public DoubleArray Capacities {
      get { return CapacitiesParameter.Value; }
      set { CapacitiesParameter.Value = value; }
    }

    #endregion

    [StorableConstructor]
    private GeneralizedQuadraticAssignmentProblem(bool deserializing) : base(deserializing) { }
    private GeneralizedQuadraticAssignmentProblem(GeneralizedQuadraticAssignmentProblem original, Cloner cloner)
      : base(original, cloner) {
      AttachEventHandlers();
    }
    public GeneralizedQuadraticAssignmentProblem()
      : base(new GQAPEvaluator(), new UniformRandomIntegerVectorCreator()) {
      Parameters.Add(new ValueParameter<DoubleMatrix>("Weights", "The weights matrix describes the flows between the equipments.", new DoubleMatrix()));
      Parameters.Add(new ValueParameter<DoubleMatrix>("Distances", "The distances matrix describes the distances between the locations at which the equipment can be installed.", new DoubleMatrix()));
      Parameters.Add(new ValueParameter<DoubleMatrix>("InstallationCosts", "The installation costs matrix describes the installation costs of installing equipment i at location j", new DoubleMatrix()));
      Parameters.Add(new FixedValueParameter<DoubleValue>("TransportationCosts", "The transportation cost represents the flow-unit per distance-unit cost factor. This value can also be set to 1 if these costs are factored into the weights matrix already.", new DoubleValue(1)));
      Parameters.Add(new ValueParameter<DoubleArray>("Demands", "The demands vector describes the space requirements of the equipments.", new DoubleArray()));
      Parameters.Add(new ValueParameter<DoubleArray>("Capacities", "The capacities vector describes the available space at the locations.", new DoubleArray()));
      Parameters.Add(new OptionalValueParameter<IItem>("BestKnownSolution", "The best known solution (if available)", null));

      Wei

      ParameterizeSolutionCreator();
      ParameterizeEvaluator();

      InitializeOperators();
      AttachEventHandlers();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new GeneralizedQuadraticAssignmentProblem(this, cloner);
    }

    #region Events
    // TODO: Add your event handlers here
    #endregion

    #region Helpers
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserializationHook() {
      AttachEventHandlers();
    }

    private void AttachEventHandlers() {
      // TODO: Add event handlers to the parameters here
    }

    private void InitializeOperators() {
      // TODO: Add custom problem analyzer to the list
      // TODO: Add operators from the representation either by direct instantiation, or by using ApplicationManager.Manger.GetInstances<T>().Cast<IOperator>()
    }
    private void ParameterizeSolutionCreator() {
      // TODO: Set the parameters of the solution creator
    }
    private void ParameterizeEvaluator() {
      // TODO: Set the parameters of the evaluator
    }
    #endregion
  }
}
